import _extends from '@babel/runtime/helpers/esm/extends';
import React from 'react';

function create(fn) {
  let listeners = [];
  let state = {
    current: fn(merge => {
      if (typeof merge === 'function') {
        merge = merge(state.current);
      }

      state.current = _extends({}, state.current, merge);
      listeners.forEach(listener => listener(state.current));
    }, () => state.current)
  };
  return [// useStore
  (selector, dependencies) => {
    let selected = selector ? selector(state.current) : state.current; // Using functional initial b/c selected itself could be a function

    const _React$useState = React.useState(() => selected),
          slice = _React$useState[0],
          set = _React$useState[1];

    React.useEffect(() => {
      const ping = () => {
        // Get fresh selected state
        let selected = selector ? selector(state.current) : state.current; // If state is not equal from the get go and not an atomic then shallow equal it

        if (slice !== selected && typeof selected === 'object' && !Array.isArray(selected)) {
          selected = Object.entries(selected).reduce((acc, _ref) => {
            let key = _ref[0],
                value = _ref[1];
            return slice[key] !== value ? _extends({}, acc, {
              [key]: value
            }) : acc;
          }, slice);
        } // Using functional initial b/c selected itself could be a function


        if (slice !== selected) {
          // Refresh local slice
          set(() => selected);
        }
      };

      listeners.push(ping);
      return () => listeners = listeners.filter(i => i !== ping);
    }, dependencies || [selector]); // Returning the selected state slice

    return selected;
  }, {
    subscribe: fn => {
      listeners.push(fn);
      return () => listeners = listeners.filter(i => i !== fn);
    },
    getState: () => state.current,
    destroy: () => {
      listeners = [];
      state.current = {};
    }
  }];
}

export default create;
